<?php
if (!defined('__TYPECHO_ROOT_DIR__')) exit;
/**
 * 用于typecho的免登录，内容付费插件，本插件制作过程中，学习参考了TeePay和editorG的思路和部分代码。
 * 
 * @package zeyu Pay
 * @author 折雨的天空
 * @version 1.0.0
 * @link https://www.144d.com
 */
class ZeyuPay_Plugin implements Typecho_Plugin_Interface
{
    /**
     * 激活插件方法,如果激活失败,直接抛出异常
     * 
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function activate()
    {
        //Typecho_Plugin::factory('admin/menu.php')->navBar = array('ZeyuPay_Plugin', 'render');
        Helper::addPanel(3, 'ZeyuPay/managePay.php', '付费记录', '管理文章付费记录', 'administrator');
        //创建数据表
        $db = Typecho_Db::get();
        self::createDB($db);
        //配置文章发布面板
        Typecho_Plugin::factory('admin/write-post.php')->bottom = array('ZeyuPay_Plugin', 'payButton');
        //设置后台文章付费金额
        Typecho_Plugin::factory('admin/write-post.php')->option = array(__CLASS__, 'setFeeForm');
        //设置文章保存时，保存付费信息
        Typecho_Plugin::factory('Widget_Contents_Post_Edit')->finishPublish = array(__CLASS__, "saveFee");
        //统计字数插件
        Typecho_Plugin::factory('Widget_Archive')->___charactersNum = array('ZeyuPay_Plugin', 'charactersNum');
        //查看文章模板内容替换
        Typecho_Plugin::factory('Widget_Archive')->___showArticle = array('ZeyuPay_Plugin', 'showArticle');
        //Typecho_Plugin::factory('Widget_Abstract_Contents')->content = array('ZeyuPay_Plugin','showArticle');
        //输出JS
        Typecho_Plugin::factory('Widget_Archive')->___payFooter = array('ZeyuPay_Plugin', 'payFooter');
        //首页摘要处理
        Typecho_Plugin::factory('Widget_Abstract_Contents')->excerptEx = array('ZeyuPay_Plugin','excerpt');
        return _t('插件已经激活成功，请配置支付aid和app secret后使用！');
    }
    
    /**
     * 禁用插件方法,如果禁用失败,直接抛出异常
     * 
     * @static
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function deactivate(){
        //删除页面模板
        Helper::removePanel(3, 'ZeyuPay/managePay.php');
        return _t('插件已被禁用');
    }
    
    /**
     * 获取插件配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form 配置面板
     * @return void
     */
    public static function config(Typecho_Widget_Helper_Form $form)
    {
        $db = Typecho_Db::get();
        $prefix = $db->getPrefix();
        //版本检查
        $div=new Typecho_Widget_Helper_Layout();
        $div->html('<small>		
			<h3>插件使用提示信息</h3>
			<span><p>第一步：申请支持支付插件，XorPay官网：<a href="https://xorpay.com/" target="_blank">https://xorpay.com</a>；</p></span>
			<span>
				<p>
					第二步：在配置文件中，配置AID和app secret。
				</p>
			</span>
			<span><p>第三步：替换模板post.php中的<code>&lt;?php $this->content(); ?></code>为：<code>&lt;?php if(isset(Typecho_Plugin::export()[\'activated\'][\'ZeyuPay\']) && !empty(Typecho_Plugin::export()[\'activated\'][\'ZeyuPay\'])){$this->showArticle();}else{$this->content();} ?></code>。</p></span>
			<span><p>第四步：在模板post.php中的底部增加代码<code>&lt;?php $this->payFooter(); ?></code>，部分模板可能导致不兼容，可以适当调整本句代码的位置。</p></span>
			<span><p>第五步：添加文章时，插入付费标签，并填写付费金额即可，付费金额为0，则为免费。</p></span>
			<span><p>注意：每篇文章仅支持一个付费标签。付费内容要放在标签之间。</p></span>
		</small>');
        $div->render();

        //xorpay设置信息配置
        $aid = new Typecho_Widget_Helper_Form_Element_Text('aid', array('value'), "", _t('AID信息'), _t('在<a href="https://xorpay.com/" target="_blank">xorpay官网</a>获取的AID。'));
        $form->addInput($aid);
        $appSecret = new Typecho_Widget_Helper_Form_Element_Text('appSecret', array('value'), "", _t('appSecret信息'), _t('在<a href="https://xorpay.com/" target="_blank">xorpay官网</a>获取的appSecret。'));
        $form->addInput($appSecret);
    }
    
    /**
     * 个人用户的配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form
     * @return void
     */
    public static function personalConfig(Typecho_Widget_Helper_Form $form){}
    
    /**
     * 插件实现方法
     * 
     * @access public
     * @return void
     */
    public static function render()
    {
        echo '<span class="message success">'
            . htmlspecialchars(Typecho_Widget::widget('Widget_Options')->plugin('ZeyuPay')->word)
            . '</span>';
    }

    /**
     * 说明: 创建数据表
     * 方法名称:createDB
     * @param $db
     */
    public static function createDB($db)
    {
        $prefix = $db->getPrefix();
        $sql[]="CREATE TABLE IF NOT EXISTS  `".$prefix."pay` (
  `pid` int(11) NOT NULL AUTO_INCREMENT,
  `bid` int(11) NOT NULL COMMENT '博客ID号',
  `money` float(10,2) DEFAULT NULL,
  `desc` text COMMENT '付费说明',
  PRIMARY KEY (`pid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='文章付费价格信息';";
        $sql[]="CREATE TABLE IF NOT EXISTS  `".$prefix."pay_log` (
  `plid` int(10) NOT NULL AUTO_INCREMENT,
  `phone` varchar(60) NOT NULL COMMENT '手机号码',
  `bid` int(10) NOT NULL COMMENT '博客ID',
  `pay_id` varchar(50) NOT NULL COMMENT '订单号',
  `pay_time` datetime DEFAULT NULL COMMENT '付款时间',
  `money` float(10,2) DEFAULT NULL COMMENT '付款金额',
  `user_info` varchar(255) DEFAULT NULL COMMENT '用户浏览器信息',
  `ip` varchar(20) DEFAULT NULL COMMENT '用户IP',
  `created` datetime DEFAULT NULL COMMENT '创建时间',
  `updated` datetime DEFAULT NULL COMMENT '更新时间',
  `status` tinyint(2) NOT NULL DEFAULT '1' COMMENT '状态，1正常，2禁用',
  `xpay_id` varchar(50) NOT NULL COMMENT 'xpay返回的订单号',
  PRIMARY KEY (`plid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='文章付费记录';";
        $sql[]="CREATE TABLE IF NOT EXISTS  `".$prefix."pay_view` (
  `vid` int(10) NOT NULL,
  `bid` int(10) DEFAULT NULL COMMENT '博客ID',
  `views` int(10) DEFAULT '0' COMMENT '查看次数',
  `phone` varchar(50) NOT NULL COMMENT '用户信息',
  `host` varchar(255) DEFAULT NULL COMMENT '用户浏览器信息',
  `ip` varchar(20) DEFAULT NULL COMMENT '用户IP信息',
  `visit_date` datetime NOT NULL COMMENT '访问时间',
  PRIMARY KEY (`vid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='付费文章查看记录';";
        foreach ($sql as $k=>$v)
        {
            $db->query($v);
        }
    }

    /**
     * 说明:设置编辑器顶部的按钮信息，改造自editorG的代码
     * 方法名称:payButton
     */
    public static function payButton(){
    ?><style>.wmd-button-row {
            height: auto;
        };
        </style>
    <script>
        $(document).ready(function(){
            $('#wmd-button-row').append('<li class="wmd-button" id="zeyuPayButton" title="插入付费内容 - 快捷键 ：ALT+X"><span style="background: none;font-size: large;text-align: center;color: orangered;font-family: serif;">￥</span></li>');
            if($('#wmd-button-row').length !== 0){
                $('#zeyuPayButton').click(function(){
                    var rs = "[zeyuPay]\n填写付费的内容\n[/zeyuPay]\n";
                    zeyuPay(rs);
                })
            }
            function zeyuPay(tag) {
                var myField;
                if (document.getElementById('text') && document.getElementById('text').type == 'textarea') {
                    myField = document.getElementById('text');
                } else {
                    return false;
                }
                //判断是否已插入标志
                if(myField.value.search('[zeyuPay]')!='-1')
                {
                    return false;
                }
                if (document.selection) {
                    myField.focus();
                    sel = document.selection.createRange();
                    sel.text = tag;
                    myField.focus();
                }
                else if (myField.selectionStart || myField.selectionStart == '0') {
                    var startPos = myField.selectionStart;
                    var endPos = myField.selectionEnd;
                    var cursorPos = startPos;
                    myField.value = myField.value.substring(0, startPos)
                        + tag
                        + myField.value.substring(endPos, myField.value.length);
                    cursorPos += tag.length;
                    myField.focus();
                    myField.selectionStart = cursorPos;
                    myField.selectionEnd = cursorPos;
                } else {
                    myField.value += tag;
                    myField.focus();
                }
            }
            $('body').on('keydown',function(a){
                if( a.altKey && a.keyCode == "88"){
                    $('#zeyuPayButton').click();
                }
            });

        });
    </script>
    <?php
    }

    /**
     * 说明: 设置后台博客发表时，付费相关输入框的信息
     * 方法名称:setFeeForm
     */
    public static function setFeeForm($post)
    {
        $db = Typecho_Db::get();
        $row = $db->fetchRow($db->select('money,desc')->from('table.pay')->where('bid = ?', $post->cid));
        $price = isset($row['money']) ? $row['money'] : '';
        $desc = isset($row['desc']) ? $row['desc'] : '';
        $html = '<section class="typecho-post-option"><label for="price" class="typecho-label">付费价格（元）</label><p><input id="price" name="price" type="text" placeholder="付费金额，留空或者为0将不会付费" value="'.$price.'" class="w-100 text"></p></section>
<section class="typecho-post-option"><label for="desc" class="typecho-label">付费说明</label><p><textarea id="desc" name="desc" type="text" placeholder="此部分说明，将被展示在支付二维码底部。" value="" class="w-100 text">'.$desc.'</textarea></p></section>';
        _e($html);
    }

    /**
     * 说明: 保存博客时，保存费用信息
     * 方法名称:saveFee
     */
    public static function saveFee($contents, $post)
    {
        $price = $post->request->get('price', 0);
        $desc = $post->request->get('desc', NULL);
        $db = Typecho_Db::get();
        if ($db->fetchObject($db->select(array('COUNT(*)' => 'num'))->from('table.pay')->where('bid = ? ', $post->cid))->num > 0) {
            $sql = $db->update('table.pay')->rows(array('money' => $price,'desc' => $desc))->where('bid=?',$post->cid);
        } else {
            $sql = $db->insert('table.pay')->rows(array('bid'=>$post->cid,'money' => $price,'desc' => $desc));
        }
        $db->query($sql);
    }

    /**
     * 说明: 前端展示付费内容
     * 方法名称:showArticle
     */
    public static function showArticle($archive)
    {
        $db = Typecho_Db::get();
        $cid = $archive->cid;
        //获取当前用户信息
        Typecho_Widget::widget('Widget_User')->to($user);
        //从数据库获取付费信息
        $fee=$db->fetchRow($db->select()->from('table.pay')->where('bid = ?', $cid ));
        if (isset($fee['money']) && $fee['money']>0 && $user->group!='administrator')
        {
            //查询用户是否付费
            $payUserId=Typecho_Cookie::get('payUserId');
            $payUserId=$payUserId?$payUserId:((isset($_COOKIE['payUserId']) && $_COOKIE['payUserId'])?$_COOKIE['payUserId']:'');
            if ($payUserId)
            {
                $userPay = $db->fetchObject($db->select(array('COUNT(*)' => 'num'))->from('table.pay_log')->where('bid = ? and phone= ? and status= ? ', $cid ,$payUserId,1))->num;
                if ($userPay<=0)
                {
                    //用户未付费
                    return self::replacePayContent($archive);
                }
            }
            else
            {
                //用户未付费
                return self::replacePayContent($archive);
            }
        }
        //已付款人数
        $payNums=$db->fetchObject($db->select(array('COUNT(*)' => 'num'))->from('table.pay_log')->where('bid = ? and status = ?', $archive->cid,1))->num;
        return str_replace(array('[zeyuPay]','[/zeyuPay]'),array('<hr /><p>以下内容为付费内容，您（当前账号：'.$payUserId.'）拥有查看权限！（本文已有<span class="layui-badge">'.$payNums.'</span>人付款）</p>','<hr />'),$archive->content);
    }

    /**
     * 说明: 付费内容替换
     * 方法名称:replacePayContent
     * @param $archive
     * @return string|string[]|null
     * @throws Typecho_Db_Exception
     * @throws Typecho_Exception
     */
    public static function replacePayContent($archive)
    {
        if(preg_match('/\[zeyuPay\](.*?)\[\/zeyuPay\]/is', $archive->content,$res))
        {
            if (isset($res[1]) && $res[1]!='')
            {
                $rescontent = '';
                $db = Typecho_Db::get();
                $options = Typecho_Widget::widget('Widget_Options');
                //获取待支付价格及描述
                $row = $db->fetchRow($db->select()->from('table.pay')->where('bid = ?', $archive->cid ));
                $row['money']=isset($row['money'])?$row['money']:0;
                $row['desc']=isset($row['desc'])?$row['desc']:'';
                if ($row['money']>0)
                {
                    //已付款人数
                    $payNums=$db->fetchObject($db->select(array('COUNT(*)' => 'num'))->from('table.pay_log')->where('bid = ? and status = ?', $archive->cid,1))->num;
                    $html='<div class="paypost">
                    <form id="payPost" onsubmit="return false" action="##" method="post">
                        <div class="feetype"><label for="feetype2"><img id="labe2" src="'.$options->pluginUrl.'/ZeyuPay/static/images/alipay2.png" nogallery="nogallery" no-zoom="true"><span>支付宝支付</span></label>
                            <input type="radio" id="feetype2" name="feetype" value="alipay" checked="checked">
                        </div>
                        <div class="cl"></div>
                        <span class="pay_price">价格： <b class="bmoney">'.$row['money'].'</b> 元 （已有<span class="layui-badge">'.$payNums.'</span>人付款）</span>
                        <div class="cl"></div>
                        <span class="pay_price pay_email"><input type="text" size="25" name="payEmail" placeholder="您的手机号码或者邮箱"></span>
                        <div class="cl"></div>
                        <input type="hidden" id="userAgent" name="userAgent" value="isDesktop" />
                        <input id="verifybtn" onclick="payBuy(\''.$archive->cid.'\',\''.$options->pluginUrl.'/ZeyuPay/pay.php?ac=pay\');" type="button" value="扫码购买" />
                    </form>
                    <div class="cl"></div>
                    <span class="tips">温馨提示：'.$row['desc'].' <br />输入邮箱或者手机号码，付款后可永久阅读隐藏的内容，请勿未经本站许可，擅自分享付费内容。<br />如果您已购买本页内容，输入购买时的手机号码或者邮箱，点击支付，即可查看内容。<br /><b>电子商品，一经购买，不支持退款，特殊原因，请联系客服。</b> </span>
                    <span class="pay_top_left">付费可读</span>
                    <span class="pay_top_right"><img src="'.$options->pluginUrl.'/ZeyuPay/static/images/icon.png" nogallery="nogallery" no-zoom="true"></span>
                </div>';
                    $rescontent = preg_replace('/\[zeyuPay\](.*?)\[\/zeyuPay\]/uis',$html,$archive->content,-1);
                }
                return $rescontent;
            }
        }
        return str_replace(array('[zeyuPay]','[/zeyuPay]'),'<hr />',$archive->content);
    }

    /**
     * 说明: 统计字数插件
     * 方法名称:charactersNum
     * @param $archive
     * @return int
     */
    public static function charactersNum($archive)
    {
        return mb_strlen($archive->text, 'UTF-8');
    }

    /**
     * 说明: 支付页面引入css和js
     * 方法名称:payFooter
     * @param $archive
     */
    public static function payFooter($archive)
    {
        $options = Typecho_Widget::widget('Widget_Options');
        echo '<link href="'.$options->pluginUrl.'/ZeyuPay/static/css/pay.css?v=1.2" type="text/css" rel="stylesheet" />';
        echo '<script src="'.$options->pluginUrl.'/ZeyuPay/static/js/jquery.min.js"></script>';
        echo '<script type="text/javascript" src="'.$options->pluginUrl.'/ZeyuPay/static/js/layer/layer.js"></script>';
        echo '<script type="text/javascript" src="'.$options->pluginUrl.'/ZeyuPay/static/js/pay.js?v=1.01"></script>';
    }

    /**
     * 说明: 替换摘要的内容
     * 方法名称:excerpt
     * @param $archive
     */
    public static function excerpt($archive)
    {
        return preg_replace('/\[zeyuPay\](.*?)\[\/zeyuPay\]/is','',$archive);
    }
}
